@@ -88,7 +88,7 @@ fieldnames: FIELDNAME ( "," FIELDNAME )*
 
 enum: [ comments ] "enum" ENUMNAME enumbody
 enumbody: "{" ( enumfield | EMPTYSTATEMENT )* "}"
-enumfield: [ COMMENTS ] IDENT "=" INTLIT [ "[" enumvalueoption ( ","  enumvalueoption )* "]" ] ";"
+enumfield: [ COMMENTS ] IDENT "=" INTLIT [ "[" enumvalueoption ( ","  enumvalueoption )* "]" ] ";" [ TAILCOMMENT ]
 enumvalueoption: OPTIONNAME "=" CONSTANT
 
 message: [ comments ] "message" MESSAGENAME messagebody
@@ -99,7 +99,7 @@ service: [ comments ] "service" SERVICENAME "{" ( option | rpc | EMPTYSTATEMENT
 rpc: [ comments ] "rpc" RPCNAME "(" [ "stream" ] MESSAGETYPE ")" "returns" "(" [ "stream" ] MESSAGETYPE ")" ( ( "{" ( googleoption | option | EMPTYSTATEMENT )* "}" ) | ";" )
 
 proto:[ comments ] syntax ( import | package | option | topleveldef | EMPTYSTATEMENT )*
-topleveldef: message | enum | service
+topleveldef: message | enum | service | comments
 
 TAILCOMMENT: [ /[^\0\n\\]/ ] COMMENT
 COMMENT: "//" /.*/ "\n"
@@ -240,15 +240,22 @@ class ProtoTransformer(Transformer):
     def enumbody(self, tokens):
         '''Returns a sequence of enum identifiers'''
         enumitems = []
-        for enumfield in tokens:
-            if enumfield.data != 'enumfield':
+        for tree in tokens:
+            if tree.data != 'enumfield':
                 continue
             comment = Comment("", {})
-            if len(enumfield.children) < 3:
-                name, value = enumfield.children
-            else:
-                comment, name, value = enumfield.children
-                comment = self.comments(comment)
+            name = Token("IDENT", "")
+            value = Token("INTLIT", "")
+            for token in tree.children:
+                if isinstance(token, Comment):
+                    comment = token
+                elif isinstance(token, Token):
+                    if token.type == "IDENT":
+                        name = token
+                    elif token.type == "INTLIT":
+                        value = token
+                    elif token.type == "COMMENTS":
+                        comment = Comment(token.value, {})
             enumitems.append(Field(comment, 'enum', 'enum', 'enum', name.value, value.value))
         return enumitems
 
